<?php
/**
 * Transaction Logs List Table.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit ; // Exit if accessed directly.
}

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php'  ;
}

if ( ! class_exists( 'WAL_Transaction_Logs_List_Table' ) ) {

	/**
	 * Class.
	 * */
	class WAL_Transaction_Logs_List_Table extends WP_List_Table {

		/**
		 * Per page count.
		 * 
		 * @var int
		 * */
		private $perpage = 20 ;

		/**
		 * Database.
		 * 
		 * @var object
		 * */
		private $database ;

		/**
		 * Wallet ID.
		 * 
		 * @var int
		 * */
		private $wallet_id ;

		/**
		 * Offset.
		 * 
		 * @var int
		 * */
		private $offset ;

		/**
		 * Order BY.
		 * 
		 * @var string
		 * */
		private $orderby = 'ORDER BY ID DESC' ;

		/**
		 * Post type.
		 * 
		 * @var string
		 * */
		private $post_type = WAL_Register_Post_Types::TRANSACTION_LOG_POSTTYPE ;

		/**
		 * List Slug.
		 * 
		 * @var string
		 * */
		private $list_slug = 'wal_transaction_logs' ;

		/**
		 * Base URL.
		 * 
		 * @var string
		 * */
		private $base_url ;

		/**
		 * Current URL.
		 * 
		 * @var string
		 * */
		private $current_url ;

		/**
		 * Constructor.
		 */
		public function __construct() {
			global $wpdb, $wal_wallet ;
			$this->database  = &$wpdb ;
			$this->wallet_id = $wal_wallet->get_id() ;

			// Prepare the required data.
			$this->base_url = wal_get_user_transaction_logs_page_url( array( 'wal_wallet_id' => $this->wallet_id ) ) ;

			parent::__construct(
					array(
						'singular' => 'transaction_log',
						'plural'   => 'transaction_logs',
						'ajax'     => false,
					)
			) ;
		}

		/**
		 * Prepares the list of items for displaying.
		 * */
		public function prepare_items() {
			//Prepare the current url.
			$this->current_url = add_query_arg( array( 'paged' => absint( $this->get_pagenum() ) ), $this->base_url ) ;

			// Prepare the perpage.
			$this->perpage = $this->get_items_per_page( 'user_transaction_logs_per_page' ) ;

			// Prepare the offset.
			$this->offset = $this->perpage * ( absint( $this->get_pagenum() ) - 1 ) ;

			// Prepare the header columns.
			$this->_column_headers = array( $this->get_columns(), $this->get_hidden_columns(), $this->get_sortable_columns() ) ;

			// Prepare the query clauses.
			$join    = $this->get_query_join() ;
			$where   = $this->get_query_where() ;
			$limit   = $this->get_query_limit() ;
			$offset  = $this->get_query_offset() ;
			$orderby = $this->get_query_orderby() ;

			// Prepare the all items.
			$count_items = $this->database->get_var( 'SELECT COUNT(DISTINCT ID) FROM ' . $this->database->posts . " AS p $where" ) ;

			// Prepare the current page items.
			$prepare_query = $this->database->prepare( 'SELECT DISTINCT ID FROM ' . $this->database->posts . " AS p $join $where $orderby LIMIT %d,%d", $offset, $limit ) ;

			$items = $this->database->get_results( $prepare_query, ARRAY_A ) ;

			// Prepare the item object.
			$this->prepare_item_object( $items ) ;

			// Prepare the pagination arguments.
			$this->set_pagination_args(
					array(
						'total_items' => $count_items,
						'per_page'    => $this->perpage,
					)
			) ;
		}

		/**
		 * Render the table.
		 * */
		public function render() {
			if ( isset( $_REQUEST[ 's' ] ) && strlen( wc_clean( wp_unslash( $_REQUEST[ 's' ] ) ) ) ) { // @codingStandardsIgnoreLine.
				/* translators: %s: search keywords */
				echo wp_kses_post( sprintf( '<span class="subtitle">' . __( 'Search results for &#8220;%s&#8221;', 'wallet-for-woocommerce' ) . '</span>', wc_clean( wp_unslash( $_REQUEST[ 's' ] ) ) ) ) ;
			}

			// Output the table.
			$this->prepare_items() ;
			$this->views() ;
			$this->display() ;
		}

		/**
		 * Get a list of columns.
		 * 
		 * @return array
		 * */
		public function get_columns() {
			$columns = array(
				'event'  => __( 'Event', 'wallet-for-woocommerce' ),
				'amount' => __( 'Amount', 'wallet-for-woocommerce' ),
				'status' => __( 'Status', 'wallet-for-woocommerce' ),
				'total'  => __( 'Available Balance', 'wallet-for-woocommerce' ),
				'date'   => __( 'Date', 'wallet-for-woocommerce' ),
					) ;

			return $columns ;
		}

		/**
		 * Get a list of hidden columns.
		 * 
		 * @return array
		 * */
		public function get_hidden_columns() {
			return array() ;
		}

		/**
		 * Get a list of sortable columns.
		 * 
		 * @return void
		 * */
		public function get_sortable_columns() {
			return array(
				'amount' => array( 'amount', false ),
				'date'   => array( 'date', false ),
				'total'  => array( 'total', false ),
					) ;
		}

		/**
		 * Message to be displayed when there are no items.
		 */
		public function no_items() {
			esc_html_e( 'No transaction log to show.', 'wallet-for-woocommerce' ) ;
		}

		/**
		 * Get a list of bulk actions.
		 * 
		 * @return array
		 * */
		protected function get_bulk_actions() {
			$action = array() ;
			/**
			 * This hook is used to alter the transaction logs bulk actions.
			 * 
			 * @since 1.0
			 */
			return apply_filters( $this->list_slug . '_bulk_actions', $action ) ;
		}

		/**
		 * Display the list of views available on this table.
		 * 
		 * @return array
		 * */
		public function get_views() {
			$args        = array() ;
			$status_link = array() ;

			$status_link_array = array(
				'all'        => __( 'All', 'wallet-for-woocommerce' ),
				'wal_credit' => __( 'Credit', 'wallet-for-woocommerce' ),
				'wal_debit'  => __( 'Debit', 'wallet-for-woocommerce' ),
					) ;

			foreach ( $status_link_array as $status_name => $status_label ) {
				$status_count = $this->get_total_item_for_status( $status_name ) ;

				if ( ! $status_count ) {
					continue ;
				}

				$args[ 'status' ] = $status_name ;

				$label = $status_label . ' (' . $status_count . ')' ;

				$class = array( strtolower( $status_name ) ) ;
				if ( isset( $_GET[ 'status' ] ) && ( sanitize_title( $_GET[ 'status' ] ) == $status_name ) ) { // @codingStandardsIgnoreLine.
					$class[] = 'current' ;
				}

				if ( ! isset( $_GET[ 'status' ] ) && 'all' == $status_name ) { // @codingStandardsIgnoreLine.
					$class[] = 'current' ;
				}

				$status_link[ $status_name ] = $this->get_edit_link( $args, $label, implode( ' ', $class ) ) ;
			}

			return $status_link ;
		}

		/**
		 * Get a edit link.
		 * 
		 * @rerurn string
		 * */
		private function get_edit_link( $args, $label, $class = '' ) {
			$url        = add_query_arg( $args, $this->base_url ) ;
			$class_html = '' ;
			if ( ! empty( $class ) ) {
				$class_html = sprintf(
						' class="%s"', esc_attr( $class )
						) ;
			}

			return sprintf(
					'<a href="%s"%s>%s</a>', esc_url( $url ), $class_html, $label
					) ;
		}

		/**
		 * Get the total item by status.
		 * 
		 * @return int
		 * */
		private function get_total_item_for_status( $status = '' ) {
			// Get the current status item ids.
			$prepare_query = $this->database->prepare( 'SELECT COUNT(DISTINCT ID) FROM ' . $this->database->posts . " WHERE post_type=%s and post_parent=%d and post_status IN('" . $this->format_status( $status ) . "')", $this->post_type, $this->wallet_id ) ;

			return $this->database->get_var( $prepare_query ) ;
		}

		/**
		 * Format the status.
		 * 
		 * @return string
		 * */
		private function format_status( $status ) {
			if ( 'all' == $status ) {
				$statuses = wal_get_transaction_log_statuses() ;
				$status   = implode( "', '", $statuses ) ;
			}

			return $status ;
		}

		/**
		 * Prepare a each column data.
		 * */
		protected function extra_tablenav( $which ) {
			if ( 'top' != $which ) {
				return ;
			}

			$export_url = add_query_arg( array( 'post_type' => $this->post_type, 'wal_export_csv' => 'transaction_logs', 'wallet_id' => $this->wallet_id ), admin_url( 'edit.php' ) ) ;
			?>
			<a href="<?php echo esc_url( $export_url ) ; ?>" class="wal-export-csv button button-primary"><?php esc_html_e( 'Export Transaction Log as CSV', 'wallet-for-woocommerce' ) ; ?></a>
			<?php
		}

		/**
		 * Prepare a each column data.
		 * */
		protected function column_default( $item, $column_name ) {                   
			switch ( $column_name ) {
							
				case 'event':
					return $item->get_formatted_event_message() ;
					break ;

				case 'amount':
					return wal_price($item->get_amount());
					break ;

				case 'status':
					return wal_display_post_status( $item->get_status() ) ;
					break ;

				case 'total':
					return wal_price($item->get_total());
					break ;

				case 'date':
					return $item->get_formatted_created_date() ;
					break ;
			}
		}

		/**
		 * Prepare the item Object.
		 * 
		 * @return void
		 * */
		private function prepare_item_object( $items ) {
			$prepare_items = array() ;
			if ( wal_check_is_array( $items ) ) {
				foreach ( $items as $item ) {
					$prepare_items[] = wal_get_transaction_log( $item[ 'ID' ] ) ;
				}
			}

			$this->items = $prepare_items ;
		}

		/**
		 * Get the query join clauses.
		 * 
		 * @return string
		 * */
		private function get_query_join() {
			$join = '' ;
			if ( empty( $_REQUEST[ 'orderby' ] ) ) { // @codingStandardsIgnoreLine.
				return $join ;
			}

			$join = ' INNER JOIN ' . $this->database->postmeta . ' AS pm ON ( pm.post_id = p.ID )' ;
			/**
			 * This hook is used to alter the query join fields.
			 * 
			 * @since 1.0
			 */
			return apply_filters( $this->list_slug . '_query_join', $join ) ;
		}

		/**
		 * Get the query where clauses.
		 * 
		 * @return string
		 * */
		private function get_query_where() {
			$current_status = 'all' ;
			if ( isset( $_GET[ 'status' ] ) && ( sanitize_title( $_GET[ 'status' ] ) != 'all' ) ) {
				$current_status = sanitize_title( $_GET[ 'status' ] ) ;
			}

			$where = " where post_type='" . $this->post_type . "' and post_status IN('" . $this->format_status( $current_status ) . "') and post_parent='" . $this->wallet_id . "'" ;
			/**
			 * This hook is used to alter the query where fields.
			 * 
			 * @since 1.0
			 */
			return apply_filters( $this->list_slug . '_query_where', $where ) ;
		}

		/**
		 * Get the query limit clauses.
		 * 
		 * @return string
		 * */
		private function get_query_limit() {
			/**
			 * This hook is used to alter the query limit fields.
			 * 
			 * @since 1.0
			 */
			return apply_filters( $this->list_slug . '_query_limit', $this->perpage ) ;
		}

		/**
		 * Get the query offset clauses.
		 * 
		 * @return string
		 * */
		private function get_query_offset() {
			/**
			 * This hook is used to alter the query offset fields.
			 * 
			 * @since 1.0
			 */
			return apply_filters( $this->list_slug . '_query_offset', $this->offset ) ;
		}

		/**
		 * Get the query order by clauses.
		 * 
		 * @return string
		 * */
		private function get_query_orderby() {
			$order = 'DESC' ;
			if ( ! empty( $_REQUEST[ 'order' ] ) && is_string( $_REQUEST[ 'order' ] ) ) { // @codingStandardsIgnoreLine.
				if ( 'ASC' === strtoupper( wc_clean( wp_unslash( $_REQUEST[ 'order' ] ) ) ) ) { // @codingStandardsIgnoreLine.
					$order = 'ASC' ;
				}
			}

			// Order By.
			if ( isset( $_REQUEST[ 'orderby' ] ) ) {
				switch ( wc_clean( wp_unslash( $_REQUEST[ 'orderby' ] ) ) ) { // @codingStandardsIgnoreLine.
					case 'amount':
						$this->orderby = " AND pm.meta_key='wal_amount' ORDER BY CAST(pm.meta_value AS DECIMAL) " . $order ;
						break ;
					case 'total':
						$this->orderby = " AND pm.meta_key='wal_total' ORDER BY CAST(pm.meta_value AS DECIMAL)  " . $order ;
						break ;
					case 'date':
						$this->orderby = ' ORDER BY p.post_date ' . $order ;
						break ;
				}
			}
			/**
			 * This hook is used to alter the query order by fields.
			 * 
			 * @since 1.0
			 */
			return apply_filters( $this->list_slug . '_query_orderby', $this->orderby ) ;
		}
	}

}
