<?php

/**
 * Current User Wallet.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit ; // Exit if accessed directly.
}
if ( ! class_exists( 'WAL_Current_User_Wallet' ) ) {

	/**
	 * Class.
	 */
	class WAL_Current_User_Wallet {

		/**
		 * User ID.
		 * 
		 * @var int
		 */
		protected static $user_id ;

		/**
		 * Wallet ID.
		 * 
		 * @var int
		 */
		protected static $wallet_id ;

		/**
		 * User name.
		 * 
		 * @var string
		 */
		protected static $user_name ;

		/**
		 * User email.
		 * 
		 * @var string
		 */
		protected static $user_email ;

		/**
		 * User.
		 * 
		 * @var object
		 */
		protected static $user ;

		/**
		 * Wallet.
		 * 
		 * @var object
		 */
		protected static $wallet ;

		/**
		 * Initialize the current user wallet details.
		 * 
		 * @return void
		 */
		public static function init() {

			self::populate_data() ;
		}

		/**
		 * Populate Data.
		 * 
		 * @return void
		 */
		public static function populate_data() {
			self::$user_id = get_current_user_id() ;

			//Retrurn if the user is guest.
			if ( ! self::$user_id ) {
				return ;
			}

			self::$wallet_id = wal_get_wallet_id_by_user_id( self::$user_id ) ;

			//Return if the user not having wallet.
			if ( ! self::$wallet_id ) {
				return ;
			}

			self::$wallet = wal_get_wallet( self::$wallet_id ) ;
		}

		/**
		 * Has user wallet?.
		 * 
		 * @return bool
		 */
		public static function has_user_wallet() {
			if ( ! self::get_wallet_id() ) {
				return false ;
			}

			if ( ! is_object( self::get_wallet() ) ) {
				return false ;
			}

			return true ;
		}

		/**
		 * Get the user.
		 * 
		 * @return object/bool.
		 */
		public static function get_user() {
			if ( self::$user ) {
				return self::$user ;
			}

			self::$user = false ;
			if ( self::get_user_id() ) {
				self::$user = get_userdata( self::get_user_id() ) ;
			}

			return self::$user ;
		}

		/**
		 * Get the user ID.
		 * 
		 * @retrun int/NULL
		 */
		public static function get_user_id() {
			return self::$user_id ;
		}

		/**
		 * Get the user name.
		 * 
		 * @return string/bool
		 */
		public static function get_user_name() {
			if ( self::$user_name ) {
				return self::$user_name ;
			}

			self::$user_name = false ;
			if ( ! is_object( self::get_user() ) ) {
				return self::$user_name ;
			}

			self::$user_name = self::get_user()->display_name ;

			return self::$user_name ;
		}

		/**
		 * Get the user email.
		 * 
		 * @return string/bool
		 */
		public static function get_user_email() {
			if ( self::$user_email ) {
				return self::$user_email ;
			}

			self::$user_email = false ;
			if ( ! is_object( self::get_user() ) ) {
				return self::$user_email ;
			}

			self::$user_email = self::get_user()->user_email ;

			return self::$user_email ;
		}

		/**
		 * Get the wallet ID.
		 * 
		 * @return int/NULL
		 */
		public static function get_wallet_id() {
			return self::$wallet_id ;
		}

		/**
		 * Get the Wallet Object.
		 * 
		 * @return object/NULL
		 */
		public static function get_wallet() {
			return self::$wallet ;
		}

		/**
		 * Get the wallet status.
		 * 
		 * @return string
		 */
		public static function get_status() {
			if ( ! self::has_user_wallet() ) {
				return 'wal_active' ;
			}

			return self::get_wallet()->get_status() ;
		}

		/**
		 * Get the wallet balance.
		 * 
		 * @return float
		 */
		public static function get_balance() {
			if ( ! self::has_user_wallet() ) {
				return 0 ;
			}
					   
			return self::get_wallet()->get_balance() ;
		}

		/**
		 * Get the wallet usage total.
		 * 
		 * @return float
		 */
		public static function get_usage_total() {
			if ( ! self::has_user_wallet() ) {
				return 0 ;
			}

			return self::get_wallet()->get_usage_total() ;
		}

		/**
		 * Get the wallet top up total.
		 * 
		 * @return float
		 */
		public static function get_topup_total() {
			if ( ! self::has_user_wallet() ) {
				return 0 ;
			}

			return self::get_wallet()->get_topup_total() ;
		}

		/**
		 * Get the wallet expired total.
		 * 
		 * @return float
		 */
		public static function get_expired_total() {
			if ( ! self::has_user_wallet() ) {
				return 0 ;
			}

			return self::get_wallet()->get_expired_total() ;
		}

		/**
		 * Get the wallet transferred total.
		 * 
		 * @return float
		 */
		public static function get_transferred_total() {
			if ( ! self::has_user_wallet() ) {
				return 0 ;
			}

			return self::get_wallet()->get_transferred_total() ;
		}

		/**
		 * Get the wallet received total.
		 * 
		 * @return float
		 */
		public static function get_received_total() {
			if ( ! self::has_user_wallet() ) {
				return 0 ;
			}

			return self::get_wallet()->get_received_total() ;
		}

		/**
		 * Get the wallet withdrew total.
		 * 
		 * @since 1.8
		 * @return float
		 */
		public static function get_withdrew_total() {
			if ( ! self::has_user_wallet() ) {
				return 0 ;
			}

			return self::get_wallet()->get_withdrew_total() ;
		}

		/**
		 * Get the wallet withdraw payment gateway.
		 * 
		 * @since 1.8
		 * @return string
		 */
		public static function get_withdrawal_payment_gateway() {
			if ( ! self::has_user_wallet() ) {
				return '' ;
			}

			return self::get_wallet()->get_withdrawal_payment_gateway() ;
		}

		/**
		 * Get the expiry date.
		 * 
		 * @return string
		 */
		public static function get_expiry_date( $format = false ) {
			if ( ! self::has_user_wallet() ) {
				return '-' ;
			}

			if ( $format ) {
				$expiry_date = self::get_wallet()->get_formatted_current_expiry_date() ;
			} else {
				$expiry_date = self::get_wallet()->get_current_expiry_date() ;
			}

			return $expiry_date ;
		}

		/**
		 * Is valid fund usage?.
		 * 
		 * @return bool
		 */
		public static function is_valid_fund_usage() {
			if ( ! self::has_user_wallet() ) {
				return false ;
			}

			// Retrun false, if the wallet status is not active.
			if ( 'wal_active' != self::get_status() ) {
				return false ;
			}

			$wallet = self::get_wallet() ;

			return WAL_Fund_Usage_Validation::is_valid( $wallet ) ;
		}

		/**
		 * Reset the properties.
		 * 
		 * @return void
		 */
		public static function reset() {
			self::$user_id    = null ;
			self::$wallet_id  = null ;
			self::$user_email = null ;
			self::$user       = null ;
			self::$wallet     = null ;

			self::populate_data() ;
		}
	}

}
