<?php

/**
 *  Handles the Checkout.
 * */
if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly.
}

if (!class_exists('WAL_Checkout_Handler')) {

	/**
	 * Class.
	 * */
	class WAL_Checkout_Handler {

		/**
		 * Class Initialization.
		 * */
		public static function init() {
			// May be render the redeem wallet fund form in the checkout.
			add_action('woocommerce_before_checkout_form', array( __CLASS__, 'maybe_render_redeem_wallet_fund_form' ), 20);
			//Handle the payment gateways.
			add_filter('woocommerce_available_payment_gateways', array( __CLASS__, 'handle_payment_gateways' ), 10, 1);
			//Handle the topup payment gateways.
			add_filter('woocommerce_available_payment_gateways', array( __CLASS__, 'handle_topup_payment_gateways' ), 10, 1);
			// Validate the checkout postdata.
			add_action('woocommerce_after_checkout_validation', array( __CLASS__, 'validate_checkout_postdata' ), 10, 2);
		}

		/**
		 * May be render the redeem wallet fund form in the checkout.
		 * 
		 * @since 1.0.0
		 * */
		public static function maybe_render_redeem_wallet_fund_form() {
			// Return if the redeem wallet fund form is not valid to display.
			if (!wal_can_render_redeem_wallet_fund_form_in_checkout()) {
				return;
			}

			wal_get_template('checkout-partial-fund.php');
		}

		/**
		 * Handles the payment gateways.
		 * 
		 * @return array.
		 */
		public static function handle_payment_gateways( $wc_gateways ) {
			if (is_null(WC()->cart) || wal_cart_topup_product_exists()) {
				return $wc_gateways;
			}

			// Return if the partial fund already applied.
			if (is_object(WC()->session) && WC()->session->get('wal_partial_fund')) {
				// Unset the wallet gateway.
				unset($wc_gateways['wal_wallet']);

				return $wc_gateways;
			}

			// Wallet having valid fund usage.
			if (WAL_Current_User_Wallet::is_valid_fund_usage()) {
				// Hide the other payment gateways.
				if ('yes' == get_option('wal_general_hide_other_wc_payment_gateways')) {
					$wallet_gateway = array( 'wal_wallet' => $wc_gateways['wal_wallet'] );
					unset($wc_gateways);

					$wc_gateways = $wallet_gateway;
				}
			} else {
				// Unset the wallet gateway, If the wallet not valid fund usage.
				unset($wc_gateways['wal_wallet']);
			}

			$balance = wal_convert_price(WAL_Current_User_Wallet::get_balance());
			if ('yes' === get_option('wal_general_hide_wallet_payment_gateway_in_checkout', 'no') && ( $balance < wal_get_wc_cart_total() )) {
				unset($wc_gateways['wal_wallet']);
			}

			return $wc_gateways;
		}

		/**
		 * Handles the topup payment gateways.
		 * 
		 * @return array.
		 */
		public static function handle_topup_payment_gateways( $wc_gateways ) {
			if (is_null(WC()->cart) || !wal_cart_topup_product_exists()) {
				return $wc_gateways;
			}

			// Remove the wallet payment gateway when purchasing the topup product.
			if (array_key_exists('wal_wallet', $wc_gateways)) {
				unset($wc_gateways['wal_wallet']);
			}

			$restricted_gateways = array_filter((array) get_option('wal_general_topup_restrict_payment_gateways'));
			if (!wal_check_is_array($restricted_gateways)) {
				return $wc_gateways;
			}

			foreach ($restricted_gateways as $gateway_id) {
				if (!isset($wc_gateways[$gateway_id])) {
					continue;
				}

				unset($wc_gateways[$gateway_id]); //unset payment gateways
			}

			return $wc_gateways;
		}

		/**
		 * Validate the checkout post data.
		 *
		 * @since 3.5.0
		 * @param  array $posted_data  An array of posted data.
		 * @param  WP_Error $errors Validation errors.
		 */
		public static function validate_checkout_postdata( $posted_data, $errors ) {
			if (!is_object(WC()->session)) {
				return;
			}

			if ('wal_wallet' !== WC()->session->get('chosen_payment_method')) {
				return;
			}
			/**
			 * This hook is used to alter the payment gateway order total.
			 *
			 * @param float WC()->cart->total
			 * @since 3.5.0 
			 */
			$total = apply_filters('wal_payment_gateway_order_total', WC()->cart->total);
			/**
			 * This hook is used to alter the payment gateway user balance.
			 *
			 * @param float WAL_Current_User_Wallet::get_balance()
			 * @since 3.5.0
			 */
			$balance = apply_filters('wal_payment_gateway_user_balance', WAL_Current_User_Wallet::get_balance());

			// Throw the error if the wallet amount less than order total. 
			if ($balance < $total) {
				$top_up = sprintf('<a href=%s>%s</a>', wal_dashboard_menu_endpoint_url('topup'), __('top-up', 'wallet-for-woocommerce'));
				/* translators: %s - topup */
				wal_add_wc_notice(sprintf(__('You don"t have sufficient funds in your wallet. Please %s the funds to complete this order.', 'wallet-for-woocommerce'), $top_up), 'error');
			}

			// Return if the current wallet not valid fund usage.
			if (!WAL_Current_User_Wallet::is_valid_fund_usage()) {
				wal_add_wc_notice(esc_html__('Sorry, You are not valid to use fund.', 'wallet-for-woocommerce'), 'error');
			}
		}
	}

	WAL_Checkout_Handler::init();
}
